use twmap::*;
use std::process::exit;
use std::ffi::OsStr;
use clap::{App, Arg};
use serde::Serialize;

#[derive(Default, Serialize)]
struct Output {
    starts: Vec<Point>,
    finishes: Vec<Point>,
}

#[derive(Serialize)]
struct Point {
    x: usize,
    y: usize,
}

fn main() {
    let matches = App::new("twmap start_finish_tiles")
        .about("Outputs coordinates of start and finish tiles, in the format y,x")
        .arg(Arg::with_name("file")
            .required(true)
            .index(1))
        .get_matches();
    match print_start_finish_tiles(matches.value_of_os("file").unwrap()) {
        Ok(output) => println!("{}", serde_json::to_string_pretty(&output).unwrap()),
        Err(err) => {
            eprintln!("{}", err);
            exit(1);
        }
    }
}

fn print_start_finish_tiles(path: &OsStr) -> Result<Output, Error> {
    let mut output = Output::default();
    let mut map = TwMap::parse_path(path)?;
    map.groups.load_conditionally(|layer| layer.kind() == LayerKind::Game)
        .map_err(|err| Error::Map(err.into()))?;
    let game_layer: &GameLayer = map.find_physics_layer().unwrap();
    game_layer.tiles.unwrap_ref().indexed_iter()
        .for_each(|((y, x), tile)| {
            match tile.id {
                33 => output.starts.push(Point{ x, y }),
                34 => output.finishes.push(Point { x, y }),
                _ => {},
            }
        });
    Ok(output)
}